require ("png");

private variable Equation_Number = 0;
private variable DVI2PNG_Pgm = tm_get_script_file ("dvi2png");

private define run_cmd (cmd)
{
   variable status = system (cmd);
   if (status != 0)
     () = fprintf (stderr, "****WARNING: %s returned %d\n", cmd, status);
}

private define add_alpha_channel (png)
{
   variable img = png_read (png) & 0x00FFFFFF;
   variable i, j;
   i = where(img != 0xFFFFFF);
   img[i] |= 0xFF000000;
   png_write (png, img, 1);
}

private define equation_function_env (esc_eq, env, opt_arg)
{
   esc_eq = strtrim (esc_eq);
   variable eq, len;
   len = strlen (esc_eq);
   if (0 == strncmp (esc_eq, "#v+", 3))
     {
	(esc_eq,) = strreplace (esc_eq, "#v+", "", len);
	(esc_eq,) = strreplace (esc_eq, "#v-", "", len);
	(esc_eq,) = strreplace (esc_eq, "#v", "", len);
	esc_eq = strtrim (esc_eq);
	eq = esc_eq;
     }
   else
     {
	(eq,) = strreplace (esc_eq, "\\\\", "\\", len);
	(eq,) = strreplace (eq, "&lt;", "<", len);
	(eq,) = strreplace (eq, "&gt;", ">", len);
	(eq,) = strreplace (eq, "&amp;", "&", len);
     }

   (opt_arg,) = strreplace (opt_arg, "\\\\", "\\", strlen(opt_arg));

   variable text = "\\documentclass[12pt]{article}\n";
   text += "\\usepackage{amsmath}\n";
   text += "\\begin{document}\n";
   text += "\\pagestyle{empty}\n";
   text += sprintf ("%s\n\\begin{%s}\n%s \\nonumber\n\\end{%s}", opt_arg, env, eq, env);
   text += "\n\\end{document}\n";
   
   Equation_Number++;
   variable tmpdir = "tmp";
   variable autopng = "autopng";

   variable base = sprintf ("eq%d", Equation_Number);
   () = mkdir (tmpdir, 0777);
   () = mkdir (autopng, 0777);
   variable tex = sprintf ("%s.tex", base);
   variable dvi = base + ".dvi";

   if (-1 == write_string_to_file (text, sprintf ("%s/%s", tmpdir, tex)))
     verror ("Write to %s failed\n", tex);
        
   run_cmd (sprintf ("cd %s; latex %s", tmpdir, tex));
   run_cmd (sprintf ("cd %s; %s %s", tmpdir, DVI2PNG_Pgm, dvi));
   variable png = sprintf ("%s/%s_eq%d.png", autopng, __TM_FILE__, Equation_Number);
   run_cmd (sprintf ("mv %s/%s.png %s", tmpdir, dvi, png));
   
   add_alpha_channel (png);

   vinsert ("<img src=\"%s\" alt=\"%s\" border=0>\n", png, esc_eq);
}

public define equation_function (esc_eq, opt_arg)
{
   equation_function_env (esc_eq, "equation*", opt_arg);
}
public define eqnarray_function (esc_eq, opt_arg)
{
   equation_function_env (esc_eq, "eqnarray*", opt_arg);
}
tm_add_macro ("equation",&equation_function, 1, 2);
tm_add_macro ("eqnarray",&eqnarray_function, 1, 2);
